create table Korisnik(
	id int not null primary key,
	ime varchar(20) not null,
	prezime varchar(30) not null
);

create table DodatneInfo(
	id int not null primary key,
	brtel varchar(15),
	imejl varchar(100),
	adresa varchar(100),
    constraint fk_korisnik foreign key (id) references Korisnik(id)
);

insert into Korisnik values
    (1, 'Filip', 'Ogrenjac'),
    (2, 'Milica', 'Obradovic');

insert into DodatneInfo values
    (1, null, 'ogrenjac.mejl', 'cerak'),
    (2, null, 'milica.mejl', 'krusevac');

create view Informacije(id, ime, prezime, telefon, imejl, adresa) as
select k.id, ime, prezime, brtel, imejl, adresa
from Korisnik k join DodatneInfo d on k.id = d.id;

select * from Informacije;

insert into Informacije values (3, 'Sava', 'Savanovic', null, null, 'Vodenica 1b');
-- Unos u pogled nastao spajanjem dve tabele je nemoguc pomocu naredbe insert nad pogledom,
-- jer SUBP ne moze automatski da zakljuci koji atribut u koju tabelu treba da smesti. Moramo koristiti
-- okidace nad pogledima.

update Informacije set imejl = 'milica.brcin@matf.bg.ac.rs' , prezime = 'Brcin' where id = 2;
-- Zelimo da azuriramo mejl i prezime osobi u pogledu Informacije.
-- Ukoliko to uradimo pomocu obicne naredbe Update, SUBP ponovo nece znati u koju tabelu
-- koju vrednost atributa da unese, tako da i u ovom slucaju moramo koristiti okidace nad pogledima.

delete from Informacije where id = 1;
-- Isto vazi i prilikom pokusaja brisanja. SUBP ne moze sam da zakljuci koja to 2 reda iz tabela
-- Korisnik i DodatneInfo treba obrisati, pa se moraju koristiti okidaci nad pogledima.

create function informacije_modify()
returns trigger as $$
begin
    if (tg_op = 'INSERT') then
        insert into Korisnik values (new.id, new.ime, new.prezime);
        insert into DodatneInfo values (new.id, new.telefon, new.imejl, new.adresa);
        return new;
    elsif (tg_op = 'UPDATE') then
        update Korisnik
        set id = new.id, ime = new.ime, prezime = new.prezime
        where id = old.id;
        update DodatneInfo
        set id = new.id, brtel = new.telefon, imejl = new.imejl, adresa = new.adresa
        where id = old.id;
        return new;
    elsif (tg_op = 'DELETE') then
        -- Moramo prvo obrisati red iz DodatneInfo zbog uslova stranog kljuca!
        delete from DodatneInfo
        where id = old.id;
        delete from Korisnik
        where id = old.id;
        return old;
    end if;
end;
$$ language plpgsql;

create trigger modifikacijeInformacija instead of insert or update or delete on Informacije
for each row
execute function informacije_modify();

insert into Informacije values (3, 'Sava', 'Savanovic', null, null, 'Vodenica 1b');
update Informacije set imejl = 'milica.brcin@matf.bg.ac.rs' , prezime = 'Brcin' where id = 2;
delete from Informacije where id = 1;

select * from Informacije;
select * from Korisnik;
select * from DodatneInfo;

drop trigger modifikacijeInformacija on Informacije;
drop function informacije_modify();
drop view Informacije;
drop table DodatneInfo;
drop table Korisnik;
